<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Reservations List Model
 *
 * @since  1.0.0
 */
class JRoomBookingModelReservations extends JModelList
{
    /**
     * Constructor.
     *
     * @param   array  $config  An optional associative array of configuration settings.
     *
     * @see     JController
     * @since   1.0.0
     */
    public function __construct($config = array())
    {
        if (empty($config['filter_fields']))
        {
            $config['filter_fields'] = array(
                'id',
                'room_name',
                'date_created',
                'start_date',
                'end_date',
                'capacity',
                'firstname',
                'surname',
                'email',
                'phone',
                'note',
                'date_approved_modified',
                'approved',
            );
        }

        parent::__construct($config);
    }

    protected function populateState($ordering = null, $direction = null)
    {
        // Initialise variables.
        $app = JFactory::getApplication('administrator');

        // Other code goes here

        $room = $app->getUserStateFromRequest($this->context . 'filter.room', 'filter_room', '', 'string');
        $this->setState('filter.room', $room);

        // Other code goes here

        // List state information.
        parent::populateState('start_date', 'asc');
    }

    public function getTable($name = 'Reservations', $prefix = 'JRoomBookingTable', $options = array())
    {
        return parent::getTable($name, $prefix, $options);
    }

    /**
     * Method to build an SQL query to load the list data.
     *
     * @return string An SQL query
     * @since 1.0.0
     */
    protected function getListQuery()
    {
        // Initialize variables.
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);

        // Create the base select statement.
        $query->select($db->qn('a') . '.*, '. $db->qn('b.name') . 'AS ' . $db->qn('room_name'))
            ->from($db->quoteName('#__jroombooking_reservations') . 'AS a')
            ->leftJoin($db->qn('#__jroombooking_rooms') . ' AS b ON (' . $db->qn('a.room_id') . ' = ' . $db->qn('b.id') . ')');

        // Filter: like / search
        $search = $this->getState('filter.search');

        if (!empty($search))
        {
            $like = $db->quote('%' . $search . '%');
            $query->where($db->quoteName('surname') . ' LIKE ' . $db->q($like)
                . ' OR ' . $db->quoteName('firstname') . ' LIKE ' . $db->q($like)
                . ' OR ' . $db->quoteName('email') . ' LIKE ' . $db->q($like)
                . ' OR ' . $db->quoteName('phone') . ' LIKE ' . $db->q($like)
                . ' OR ' . $db->quoteName('note') . ' LIKE ' . $db->q($like));
        }

        // Filter by room
        $room = $db->escape($this->getState('filter.room'));
        if (!empty($room)) {
            $query->where($db->qn('room_id') . '=' . (int)$room);
        }

        // Filter by published state
        $published = $this->getState('filter.published');

        if (is_numeric($published))
        {
            $query->where($db->quoteName('approved') . '=' . $db->q((int) $published));
        }
        elseif ($published === '')
        {
            $query->where('(' . $db->quoteName('approved') . ' IN (0, 1))');
        }

        // Add the list ordering clause.
        $app = JFactory::getApplication();
        $context = "com_jroombooking.reservations.list";
        $orderCol 	= $app->getUserStateFromRequest($context.'.filter_order', 'filter_order', 'id', 'cmd');
        $orderDirn = $app->getUserStateFromRequest($context.'.filter_order_Dir', 'filter_order_Dir', 'asc', 'cmd');

        $query->order($db->escape($orderCol) . ' ' . $db->escape($orderDirn));

        return $query;
    }
}