<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Rooms List Model
 *
 * @since  1.0.0
 */
class JRoomBookingModelRooms extends JModelList
{
    /**
     * Constructor.
     *
     * @param   array  $config  An optional associative array of configuration settings.
     *
     * @see     JController
     * @since   1.6
     */
    public function __construct($config = array())
    {
        if (empty($config['filter_fields']))
        {
            $config['filter_fields'] = array(
                'id',
                'photo',
                'name',
                'description',
                'price',
                'max_capacity',
                'published'
            );
        }

        parent::__construct($config);
    }

    /**
     * Method to build an SQL query to load the list data.
     *
     * @return string An SQL query
     * @since 1.0.0
     */
    protected function getListQuery()
    {
        // Initialize variables.
        $db    = JFactory::getDbo();
        $query = $db->getQuery(true);

        // Create the base select statement.
        $query->select('*')
            ->from($db->quoteName('#__jroombooking_rooms'));

        // Filter: like / search
        $search = $this->getState('filter.search');

        if (!empty($search))
        {
            $like = $db->quote('%' . $search . '%');
            $query->where($db->qn('name') . ' LIKE ' . $like);
        }

        // Filter by published state
        $published = $this->getState('filter.published');

        if (is_numeric($published))
        {
            $query->where($db->qn('published') . ' = ' . $db->q((int) $published));
        }
        elseif ($published === '')
        {
            $query->where('(' . $db->qn('published') . ' IN (0, 1))');
        }

        // Add the list ordering clause.
        $app = JFactory::getApplication();
        $context = "com_jroombooking.rooms.list";
        $orderCol 	= $app->getUserStateFromRequest($context.'.filter_order', 'filter_order', 'id', 'cmd');
        $orderDirn = $app->getUserStateFromRequest($context.'.filter_order_Dir', 'filter_order_Dir', 'asc', 'cmd');

        $query->order($db->escape($orderCol) . ' ' . $db->escape($orderDirn));

        return $query;
    }
}