<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');
/**
 * JRoomBooking Component Controller
 *
 * @since  1.0.0
 */
class JRoomBookingController extends JControllerLegacy
{
    /**
     * Akce vykonana pri stisknuti tlacitka 'zrusit'. Smaze data o rezervaci ze session a presmeruje na hlavni stranku komponenty.
     *
     * @return bool
     * @since 1.0.0
     */
    public function cancel() {
        $app = JFactory::getApplication();
        $app->setUserState('com_jroombooking.reserverequest.data', null);
        $app->setUserState('com_jroombooking.reserveapply.data', null);
        $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&', false));

        return true;
    }

    /**
     * Akce vykonana pri stisknuti tlacitka 'rezervovat'. Zkontroluje, zda je v administraci nastaven alespon jeden e-mail spravce.
     * Pokud neni, na front-endu se vypise chyba a uzivatel je presmerovan zpet.
     *
     * @since 1.0.0
     */
    public function reserve() {
        if (!$this->getRecipients()) {
            $app	= JFactory::getApplication();
            $app->enqueueMessage(JText::_('COM_JROOMBOOKING_CONFIG_EMAIL_ERROR'), 'error');
            $this->setRedirect(JRoute::_('index.php?option=' . $this->input->getString('option'), false));
        }
        $this->display();
    }

    private function getRecipients() {
        $params = JComponentHelper::getParams('com_jroombooking');
        $list_recipients = $params->get('emails', '');
        //parsovani emailu
        $recipients = explode(",", $list_recipients);
        for ($i = 0, $iMax = count($recipients); $i < $iMax; $i++) {
            //odstraneni whitespace
            $recipients[$i] = trim($recipients[$i]);
            //odstraneni prazdnych retezcu
            if ($recipients[$i] == '') unset($recipients[$i]);
        }
        return count($recipients) > 0;
    }

    /**
     * Metoda spustena po odeslani formulare pro filtraci pokoju.
     *
     * @return bool vysledek vyhodnoceni
     * @throws Exception chyba modelu
     * @since 1.0.0
     */
    public function filterDate() {
        // Check for request forgeries
        JSession::checkToken() or die(JText::_('JINVALID_TOKEN'));

        // Initialise variables.
        $app	= JFactory::getApplication();
        $model	= $this->getModel('jroombooking');
        $form = $model->getFilterForm();
        $input = $app->input;

        if (!$form)
        {
            throw new Exception($model->getError, 500);
        }

        // Get the data from the form POST
        $data = array();
        $data['start_date'] = '';
        $data['end_date'] = '';
        $data['capacity'] = '';
        $data = $input->post->getArray($data);

        // Save the data in the session.
        $app->setUserState('com_jroombooking.jroombooking.filterdate.data', $data);

        try {
            $model->validate($data);
        } catch (Exception $e) {
            $app->enqueueMessage($e->getMessage(), 'error');
            $this->setRedirect(JRoute::_('index.php?option=com_jroombooking', false));
            return false;
        }

        $app->setUserState('com_jroombooking.reserverequest.data', $data);
        $this->setRedirect(JRoute::_('index.php?option=com_jroombooking', false));
        return true;
    }

    /**
     * Metoda pro zruseni filtrce pokoju. Smaze udaje pro filtrovani ze session a presmeruje na stranku s pokoji.
     *
     * @return bool vysledek vyhodnoceni
     * @since 1.0.0
     */
    public function cancelFilterDate() {
        $app = JFactory::getApplication();
        $app->setUserState('com_jroombooking.jroombooking.filterdate.data', null);
        $this->setRedirect(JRoute::_('index.php?option=com_jroombooking', false));

        return true;
    }
}