<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// Include dependancy of the main controllerform class
jimport('joomla.application.component.controllerform');

/**
 * Class JRoomBookingControllerReserveApply
 *
 * @since 1.0.0
 */
class JRoomBookingControllerReserveApply extends JControllerForm
{
    public function getModel($name = '', $prefix = '', $config = array('ignore_request' => true))
    {
        return parent::getModel($name, $prefix, $config);
    }

    /**
     * Akce spustena po odeslani zadosti o rezervaci. Data z formulare jsou validovana.
     * Pokud jsou v poradku, rezervace se ulozi do DB a spravcum je na e-mail odeslana notifikace
     *
     * @return bool Vysledek akce
     * @throws Exception
     * @since 1.0.0
     */
    public function submit() {
        // Check for request forgeries
        JSession::checkToken() or die(JText::_('JINVALID_TOKEN'));

        // Initialise variables.
        $app	= JFactory::getApplication();
        $model	= $this->getModel('reserveapply');
        $form = $model->getForm();

        if (!$form)
        {
            throw new Exception($model->getError, 500);
        }

        // Get the data from the form POST
        $data = $this->input->post->get('jform', array(), 'array');

        // Save the data in the session.
        $app->setUserState('com_jroombooking.reserveapply.data', $data);

        $modelData = $model->getData();
        $roomId1 = $modelData->room_id;

        $stub = $this->input->getString('room_id');
        $roomId2 = (int)$stub;

        //id pokoje z predchoziho formulare se musi shodovat s tim, co je v URL
        if ($roomId1 === $roomId2) {

            $validate = $model->validate($form, $data);
            if ($validate === false)
            {
                // Get the validation messages.
                $errors = $model->getErrors();

                // Push up to three validation messages out to the user.
                for ($i = 0, $n = count($errors); $i < $n && $i < 3; $i++)
                {
                    if ($errors[$i] instanceof Exception)
                    {
                        $app->enqueueMessage($errors[$i]->getMessage(), 'error');
                    }
                    else
                    {
                        $app->enqueueMessage($errors[$i], 'error');
                    }
                }

                // Redirect back to the contact form.
                $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&view=reserveapply&room_id=' . $stub, false));

                return false;
            }

            $startDate =  new JDate($modelData->start_date);
            $startDate = $startDate->format('Y-m-d');
            $endDate =  new JDate($modelData->end_date);
            $endDate = $endDate->format('Y-m-d');
            $validDates = JModelLegacy::getInstance('ReserveRequest', 'JRoomBookingModel')->validateDates($startDate, $endDate);
            if (!$validDates) {
                $app->enqueueMessage(JText::_('COM_JROOMBOOKING_RESERVE_ROOM_OCCUPIED'), 'error');
                $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&view=reserveapply&room_id=' . $stub, false));
                return false;
            }

            // ziskani seznamu prijemcu notifikacniho mailu
            $recipients = $this->getRecipients();
            if ($recipients === false) {
                $app->enqueueMessage(JText::_('COM_JROOMBOOKING_CONFIG_EMAIL_ERROR'), 'error');
                $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&view=reserveapply&room_id=' . $stub, false));
                return false;
            }

            //ulozeni pozadavku na rezervaci do DB
            $insert = $model->saveData();
            if ($insert === false) {
                // vlozeni do DB se nezdarilo
                $app->enqueueMessage(JText::_('COM_JROOMBOOKING_RESERVE_FORM_ERROR'), 'error');
                $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&view=reserveapply&room_id=' . $stub, false));
                return false;
            } else {
                //odeslani emailu spravci
                $this->sendNotifications($recipients);

                // odeslani mailu zadavateli rezervace, pokud je to zapnuto v nastaveni
                if(JComponentHelper::getParams('com_jroombooking')->get('requested_use_email') === "1")
                {
                  $requestedSend = $this->sendRequestedEmail($insert);
                  if ($requestedSend === false) {
                    //chyba pri odesilani zpravy uzivateli
                    $app->enqueueMessage(JText::_('COM_JROOMBOOKING_RESERVATIONS_ERROR_SENDING_EMAIL'), 'error');
                  }
                }

                // uspesne vlozeno
                // smazani dat ze session
                $app->setUserState('com_jroombooking.reserverequest.data', null);
                $app->setUserState('com_jroombooking.reserveapply.data', null);

                //zobrazeni hlasky a presmerovani
                $app->enqueueMessage(JText::_('COM_JROOMBOOKING_RESERVE_FORM_SUCCESS'), 'message');
                $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&', false));
                return true;
            }

        } else {
            //chybova hlaska, presmerovani na prvni formular rezervace
            $app->enqueueMessage(JText::_('COM_JROOMBOOKING_RESERVE_FORM_ERROR'), 'error');
            $this->setRedirect(JRoute::_('index.php?option=com_jroombooking&view=reserverequest&room_id=' . $stub, false));

            return false;
        }
    }

    /**
     * Odesle email spravci (spravcum) v pripade nove rezervace,
     */
    private function sendNotifications($recipients) {
        $mailer = JFactory::getMailer();
        $model = $this->getModel('reserveapply');
        $data = $model->getData();
        $roomName = $model->getRoom()->name;

        $config = JFactory::getConfig();
        $sender = array(
            $config->get( 'mailfrom' ),
            $config->get( 'fromname' )
        );

        $mailer->setSender($sender);
        $mailer->addRecipient($recipients);

      // Nastavi typ zpravy na HTML
      // $mailer->isHtml(true);

        $mailer->setSubject(JText::_('COM_JROOMBOOKING_EMAIL_SUBJECT_NEW_RESERVATION'));
        $body = JText::_('COM_JROOMBOOKING_EMAIL_BODY_TERM') . $data->start_date . ' - ' . $data->end_date . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_ROOM_NAME') . $roomName . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_ROOM_ID') . $data->room_id . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_CAPACITY') . $data->capacity . PHP_EOL . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_FIRSTNAME') . $data->firstname . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_SURNAME') . $data->surname . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_EMAIL') . $data->email . PHP_EOL
            . JText::_('COM_JROOMBOOKING_EMAIL_BODY_PHONE') . $data->phone;
        if ($data->note != '') {
            $body .= PHP_EOL . JText::_('COM_JROOMBOOKING_EMAIL_BODY_NOTE') . $data->note;
        }
        $mailer->setBody($body);

        $send = $mailer->Send();
        return $send;
    }

    private function getRecipients() {
        $params = JComponentHelper::getParams('com_jroombooking');
        $list_recipients = $params->get('emails', '');
        //parsovani emailu
        $recipients = explode(",", $list_recipients);
        for ($i = 0, $iMax = count($recipients); $i < $iMax; $i++) {
            //odstraneni whitespace
            $recipients[$i] = trim($recipients[$i]);
            //odstraneni prazdnych retezcu
            if ($recipients[$i] == '') unset($recipients[$i]);
        }

        if (count($recipients) > 0) {
            return $recipients;
        } else {
            //nastala chyba, neni nastaven zadny prijemce
            return false;
        }
    }

    /**
     * Odesle email uzivateli s informaci o vytvoreni nove rezervace (neschvalene).
     */
    private function sendRequestedEmail($id) {
      $mailer = JFactory::getMailer();
      $model = $this->getModel('reserveapply');
      $data = $model->getReservation($id);
      $params =  JComponentHelper::getParams('com_jroombooking');
      $config = JFactory::getConfig();
      $sender = array(
        $config->get( 'mailfrom' ),
        $config->get( 'fromname' )
      );
      // Nastavi typ zpravy na HTML
      $mailer->isHtml(true);
      $mailer->setSender($sender);
      $mailer->addRecipient($data->email);
      $mailer->setSubject($params->get('requested_email_subject', 'Room reservations - Reservation requested'));

      $startDate = new JDate($data->start_date);
      $startDate = $startDate->format(JText::_('COM_JROOMBOOKING_RESERVATIONS_SHORT_DATE_FORMAT'));

      $endDate = new JDate($data->end_date);
      $endDate = $endDate->format(JText::_('COM_JROOMBOOKING_RESERVATIONS_SHORT_DATE_FORMAT'));

      $textColor = $params->get('requested_email_text_color', '#ff6633');

      $logo = $params->get('requested_email_logo');
      // pokud neni logo nastaveno, pouzije se vychozi
      $logoSrc = $logo === null ? 'https://www.jroombooking.com/images/logo.png' : (JURI::root() . 'images/' . $logo);

      $body  = '<div style="text-align: center">';
      $body .= '<a href="'.JURI::root().'"><img src="'.$logoSrc.'" style="width: 400px; height: auto; margin: 0 auto;"></a>';
      $body .= '<h1 style="margin: 50px 0;">' . $params->get('requested_email_body', 'Your reservation has been requested. Wait for approval.') . '</h1>';
      $body .= '<h2 style="line-height: 1;">Reservation information:</h2>';

      $body .= '<p style="margin-bottom: 3px">' . $params->get('requested_email_name', 'Name') . '</p>';
      $body .= '<p style="margin-top: 0; color: '.$textColor.'; font-size: 24px; font-weight: bold; line-height: 1;">' . $data->firstname . ' ' . $data->surname . '</p>';
      $body .= '<p style="margin-bottom: 3px">' . $params->get('requested_email_term', 'Term of booking') . '</p>';
      $body .= '<p style="margin-top: 0; color: '.$textColor.'; font-size: 24px; font-weight: bold; line-height: 1;">' . $startDate . ' - ' . $endDate . '</p>';
      $body .= '<p style="margin-bottom: 3px">' . $params->get('requested_email_room_name', 'Room name') . '</p>';
      $body .= '<p style="margin-top: 0; color: '.$textColor.'; font-size: 24px; font-weight: bold; line-height: 1;">' . $data->name . '</p>';
      $body .= '<p style="margin-bottom: 3px">' . $params->get('requested_email_capacity', 'Number of beds') . '</p>';
      $body .= '<p style="margin-top: 0; color: '.$textColor.'; font-size: 24px; font-weight: bold; line-height: 1;">' . $data->capacity . '</p>';
      $body .= '</div>';

      $mailer->setBody($body);

      $send = $mailer->Send();
      return $send;
    }
}