<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * ReserveRequest Model
 *
 * @since  1.0.0
 */
class JRoomBookingModelReserveApply extends JModelForm
{
    protected $data;
    protected $room;

    /**
     * Metoda pro ziskani mistnosti.
     *
     * @return mixed
     * @since 1.0.0
     */
    public function getRoom()
    {
        if (!isset($this->room)) {
            $this->room = JModelLegacy::getInstance('ReserveRequest', 'JRoomBookingModel')->getRoom();
        }

        return $this->room;
    }

    /**
     * Method to get the record form.
     *
     * @param   array    $data      Data for the form.
     * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
     *
     * @return  mixed    A JForm object on success, false on failure
     *
     * @since   1.0.0
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm(
            'com_jroombooking.reserveapply',
            'reserveapply',
            array(
                'control' => 'jform',
                'load_data' => $loadData
            )
        );

        if (empty($form))
        {
            return false;
        }

        return $form;
    }

    /**
     * Method to get the registration form data.
     *
     * The base form data is loaded and then an event is fired
     * for users plugins to extend the data.
     *
     * @return  mixed  Data object on success, false on failure.
     *
     * @since   1.0.0
     */
    public function getData()
    {

        $this->data = new stdClass;
        $app = JFactory::getApplication();

        // Override the base user data with any data in the session.
        $temp = (array) $app->getUserState('com_jroombooking.reserveapply.data', array());
        $temp2 = (array) $app->getUserState('com_jroombooking.reserverequest.data', array());
        $temp = array_merge($temp, $temp2);

        foreach ($temp as $k => $v)
        {
            $this->data->$k = $v;
        }


        return $this->data;
    }


    /**
     * Method to get the data that should be injected in the form.
     *
     * @return  mixed  The data for the form.
     *
     * @since   1.0.0
     */
    protected function loadFormData()
    {
        $data = $this->getData();

        return $data;
    }

    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @return  void
     *
     * @since   1.0.0
     */
    protected function populateState()
    {
        // Get the application object.
        $app = JFactory::getApplication();
        $params = $app->getParams('com_jroombooking');

        // Load the parameters.
        $this->setState('params', $params);
    }

    /**
     * Method to get the script that have to be included on the form
     *
     * @return string	Script files
     * @since 1.0.0
     */
    public function getScript()
    {
        return 'components/com_jroombooking/models/forms/reserveapply.js';
    }

    /**
     * Metoda pro ulozeni dat do DB.
     *
     * @return mixed id vytvorene rezervace nebo false
     * @since 1.0.0
     */
    public function saveData() {
        $reservation = $this->getData();
        $startDate = new JDate($reservation->start_date);
        $endDate = new JDate($reservation->end_date);
        $reservation->start_date = $startDate->format('Y-m-d');
        $reservation->end_date = $endDate->format('Y-m-d');
        $nowDate = new JDate();
        $nowDate = $nowDate->toSql();
        //ziskani jazyka uzivatele
        $language = JFactory::getUser()->getParam('language', JFactory::getLanguage()->getTag());

        $db = JFactory::getDbo();

        $query = $db->getQuery(true);
        $columns = array('room_id', 'date_created', 'date_modified', 'start_date', 'end_date',
            'capacity', 'firstname', 'surname', 'email', 'phone',
            'note', 'language', 'date_approved_modified');
        $values = array($reservation->room_id, $nowDate, $nowDate, $reservation->start_date, $reservation->end_date,
            $reservation->capacity, $reservation->firstname, $reservation->surname, $reservation->email,
            $reservation->phone, $reservation->note, $language, $nowDate);
        $query->insert($db->qn('#__jroombooking_reservations'))->columns($db->qn($columns))->values((implode(',', $db->q($values))));
        $db->setQuery($query);
        $result = $db->execute();

        return $result ? $db->insertid() : false;
    }

    /**
     * Ziska data rezervace pro zadane ID rezervace.
     *
     * @param int $id ID rezervace
     * @return mixed Objekt s daty
     * @since 1.0.0
     */
    public function getReservation($id) {
      $db = JFactory::getDbo();
      $query = $db->getQuery(true);
      $query->select($db->qn(array('a.id', 'a.email', 'a.firstname', 'a.surname', 'a.start_date', 'a.end_date', 'a.capacity', 'b.name')))
        ->from($db->qn('#__jroombooking_reservations') . ' AS a')
        ->leftJoin($db->qn('#__jroombooking_rooms') . ' AS b ON (' . $db->qn('a.room_id') . ' = ' . $db->qn('b.id') . ')')
        ->where($db->qn('a.id') . ' = ' . $db->q($id));
      $db->setQuery($query);
      $email = $db->loadObject();
      return $email;
    }
}