<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// import the Joomla modellist library
jimport('joomla.application.component.modellist');

/**
 * JRoomBooking Model
 *
 * @since  1.0.0
 */
class JRoomBookingModelJRoomBooking extends JModelList
{
    /**
     *
     *
     * @var object item
     * @since 1.0.0
     */
    protected $item;
    protected $rooms;
    protected $availableRoomsCount;
    protected $filteredRoomsCount;
    protected $params;

    /**
     * Method to auto-populate the model state.
     *
     * This method should only be called once per instantiation and is designed
     * to be called on the first call to the getState() method unless the model
     * configuration flag to ignore the request is set.
     *
     * Note. Calling getState in this method will result in recursion.
     * @param   string  $ordering   An optional ordering field.
     * @param   string  $direction  An optional direction (asc|desc).
     * @return    void
     * @since    1.0.0
     */
    protected function populateState($ordering = null, $direction = null)
    {
        //pagination
        $jinput = JFactory::getApplication()->input;
        $limit = $jinput->get('limit', JComponentHelper::getParams('com_jroombooking')->get('rooms_list_limit', 10), 'uint');
        $limitstart = $jinput->get('limitstart', 0, 'uint');
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);

        $search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
        $this->setState('filter.search', $search);

        // Load the parameters.
        $this->setState('params', JFactory::getApplication()->getParams());
        parent::populateState($ordering, $direction);
    }

    /**
     * Method to build an SQL query to load the list data.
     *
     * @return    string    An SQL query
     * @since 1.0.0
     */
    protected function getListQuery()
    {
        // Create a new query object.
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);
        $query->select($db->qn(array('id', 'name', 'description', 'photo', 'max_capacity', 'published')))
            ->from($db->qn('#__jroombooking_rooms'))
            ->where($db->qn('published') . ' = ' . $db->q(1))
            ->order($db->qn('name') . ' ASC');

        // Filter: like / search
        $search = $this->getState('filter.search');

        if (!empty($search))
        {
            $like = $db->quote('%' . $search . '%');
            $query->where($db->qn('name') . ' LIKE ' . $like);
        }

        // Filter: rooms available in given date and capacity
        $filterDate = JFactory::getApplication()->getUserState('com_jroombooking.jroombooking.filterdate.data', array());
        if (!empty($filterDate)) {
            $startDate = new JDate($filterDate['start_date']);
            $startDate = $startDate->toSql();
            $endDate = new JDate($filterDate['end_date']);
            $endDate = $endDate->toSql();
            $capacity = $filterDate['capacity'];

            $query->where($db->qn('max_capacity') . ' >= ' . $db->q($capacity));

            $subQuery = $db->getQuery(true);
            $subQuery->select($db->qn('room_id'))
                ->from($db->qn('#__jroombooking_reservations'))
                ->where($db->qn('approved') . ' = ' . $db->q(1)
                    . ' AND ((' . $db->qn('start_date') . ' >= ' . $db->q($startDate)
                    . ' AND ' . $db->qn('end_date') . ' <= ' . $db->q($endDate) . ') OR (' . $db->qn('end_date') . ' >= ' . $db->q($startDate)
                    . ' AND ' . $db->qn('start_date') . ' <= '. $db->q($endDate) .'))');

            $query->where($db->qn('id') . ' NOT IN (' . $subQuery . ')');
        }

        return $query;
    }

    /**
     * Method to get a table object, load it if necessary.
     *
     * @param   string $type The table name. Optional.
     * @param   string $prefix The class prefix. Optional.
     * @param   array $config Configuration array for model. Optional.
     *
     * @return  JTable  A JTable object
     *
     * @since   1.0.0
     */
    public function getTable($type = 'JRoomBooking', $prefix = 'JRoomBookingTable', $config = array())
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    public function getParams()
    {
        if (!isset($this->params)) {
            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->select($db->qn('params'))
                ->from($db->qn('#__jroombooking'));
            $db->setQuery((string)$query);
            $this->params = $db->loadObjectList();

            if ($this->params = $db->loadObject()) {
                // Load the JSON string
                $params = new JRegistry;
                $params->loadString($this->params, 'JSON');
                $this->params = $params;

                // Merge global params with item params
                $params = clone $this->getState('params');
                $params->merge($this->params);
                $this->params = $params;
            }
        }
        return $this->params;
    }

    /**
     * Ziska pocet zverejnenych pokoju.
     *
     * @return int Pocet zverejnenych pokoju
     * @since 1.0.0
     */
    public function getAvailableRoomsCount()
    {
        if (!isset($this->availableRoomsCount)) {
            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->select('COUNT(id) pocet_celkem')
                ->from($db->qn('#__jroombooking_rooms'))
                ->where($db->qn('published') . ' = 1');
            $db->setQuery($query);
            $this->availableRoomsCount = $db->loadResult();
        }

        return $this->availableRoomsCount;
    }

    /**
     * Ziska pocet filtrovanych pokoju.
     *
     * @return int Pocet filtrovanych pokoju
     * @since 1.0.0
     */
    public function getFilteredRoomsCount()
    {
        if (!isset($this->filteredRoomsCount)) {
            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->select('COUNT(id) pocet_celkem')
                ->from($db->qn('#__jroombooking_rooms'))
                ->where($db->qn('published') . ' = 1');

            // Filter: rooms available in given date and capacity
            $filterDate = JFactory::getApplication()->getUserState('com_jroombooking.jroombooking.filterdate.data', array());
            if (!empty($filterDate)) {
                $startDate = new JDate($filterDate['start_date']);
                $startDate = $startDate->toSql();
                $endDate = new JDate($filterDate['end_date']);
                $endDate = $endDate->toSql();
                $capacity = $filterDate['capacity'];

                $query->where($db->qn('max_capacity') . ' >= ' . $db->q($capacity));

                $subQuery = $db->getQuery(true);
                $subQuery->select($db->qn('room_id'))
                    ->from($db->qn('#__jroombooking_reservations'))
                    ->where($db->qn('approved') . ' = ' . $db->q(1)
                        . ' AND ((' . $db->qn('start_date') . ' >= ' . $db->q($startDate)
                        . ' AND ' . $db->qn('end_date') . ' <= ' . $db->q($endDate) . ') OR (' . $db->qn('end_date') . ' >= ' . $db->q($startDate)
                        . ' AND ' . $db->qn('start_date') . ' <= '. $db->q($endDate) .'))');

                $query->where($db->qn('id') . ' NOT IN (' . $subQuery . ')');
            }

            $db->setQuery($query);
            $this->filteredRoomsCount = $db->loadResult();
        }

        return $this->filteredRoomsCount;
    }

    /**
     * Ziska text nadpisu komponenty.
     *
     * @return mixed
     * @since 1.0.0
     */
    public function getHeadingText() {
        return JComponentHelper::getParams('com_jroombooking')->get('heading_text', JText::_('COM_JROOMBOOKING_TITLE'));
    }

    /**
     * Ziska zverejnene mistnosti.
     *
     * @return array Pole se zverejnenymi mystnostmi
     * @since 1.0.0
     */
    public function getRooms()
    {
        if (!isset($this->rooms)) {
            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->select($db->qn(array('id', 'name', 'description', 'photo', 'max_capacity', 'published')))
                ->from($db->qn('#__jroombooking_rooms'))
                ->where($db->qn('published') . ' = 1')
                ->order('name ASC');

            // Filter: rooms available in given date and capacity
            $filterDate = JFactory::getApplication()->getUserState('com_jroombooking.jroombooking.filterdate.data', array());
            if (!empty($filterDate)) {
                $startDate = new JDate($filterDate['start_date']);
                $startDate = $startDate->toSql();
                $endDate = new JDate($filterDate['end_date']);
                $endDate = $endDate->toSql();
                $capacity = $filterDate['capacity'];

                $query->where($db->qn('max_capacity') . ' >= ' . $db->q($capacity));

                $subQuery = $db->getQuery(true);
                $subQuery->select($db->qn('room_id'))
                    ->from($db->qn('#__jroombooking_reservations'))
                    ->where($db->qn('approved') . ' = ' . $db->q(1)
                        . ' AND ((' . $db->qn('start_date') . ' >= ' . $db->q($startDate)
                        . ' AND ' . $db->qn('end_date') . ' <= ' . $db->q($endDate) . ') OR (' . $db->qn('end_date') . ' >= ' . $db->q($startDate)
                        . ' AND ' . $db->qn('start_date') . ' <= '. $db->q($endDate) .'))');

                $query->where($db->qn('id') . ' NOT IN (' . $subQuery . ')');
            }

            $db->setQuery($query, $this->getState('limitstart'), $this->getState('limit'));
            $this->rooms = $db->loadAssocList('id');
        }
        return $this->rooms;
    }

    /**
     * Method to get a JPagination object for the data set.
     *
     * @return  JPagination  A JPagination object for the data set.
     * @since   1.0.0
     */
    public function getPagination()
    {
        // Get a storage key.
        $store = $this->getStoreId('getPagination');

        // Try to load the data from internal storage.
        if (isset($this->cache[$store]))
        {
            return $this->cache[$store];
        }

        // Create the pagination object.
        $limit = (int) $this->getState('limit') - (int) $this->getState('list.links');
        $page = new JPagination($this->getTotal(), $this->getStart(), $limit);

        // Add the object to the internal cache.
        $this->cache[$store] = $page;

        return $this->cache[$store];
    }

    /**
     * Method to get the starting number of items for the data set.
     *
     * @return  integer  The starting number of items available in the data set.
     *
     * @since   1.0.0
     */
    public function getStart()
    {
        $store = $this->getStoreId('getstart');

        // Try to load the data from internal storage.
        if (isset($this->cache[$store]))
        {
            return $this->cache[$store];
        }

        $start = $this->getState('limitstart');
        $limit = $this->getState('limit');

        if ($start > 0)
        {
            $total = $this->getTotal();
            if ($start > $total - $limit)
            {
                $start = max(0, (int) (ceil($total / $limit) - 1) * $limit);
            }
        }

        // Add the total to the internal cache.
        $this->cache[$store] = $start;

        return $this->cache[$store];
    }

    /**
     * Validuje formular pro filtraci pokoju podle terminu a kapacity.
     *
     * @param array $data data z formulare
     * @return bool vysledek validace
     * @throws Exception vyjimka, pokud neco neni validni
     * @since 1.0.0
     */
    public function validate($data) {
        if ($data['start_date'] == '') {
            throw new Exception(JText::_('COM_JROOMBOOKING_FILTER_FROM_DATE_EMPTY'));
        }

        if ($data['end_date'] == '') {
            throw new Exception(JText::_('COM_JROOMBOOKING_FILTER_TO_DATE_EMPTY'));
        }

        $startDate = new JDate($data['start_date']);
        $endDate = new JDate($data['end_date']);
        //zacatek rezervace  = dnesek + X dni; X = 1
        $earliest = 1;
        $signum = $earliest >= 0 ? '+' : '-';
        $earliestDate = new JDate('NOW ' . $signum . ' ' . abs($earliest) . ' DAY');

        //kontrola, jestli neni datum zacatku rezervace starsi nez dnesek
        if ($startDate->format('Y-m-d') < $earliestDate->format('Y-m-d')) {
            throw new Exception(JText::_('COM_JROOMBOOKING_RESERVE_START_DATE_LESS_OR_EQUAL_NOW') . $earliestDate->format(JText::_('COM_JROOMBOOKING_DATE_FORMAT'), false, true));
        }

        //datum zacatku musi byt drive nez datum konce
        if ($startDate >= $endDate) {
            throw new Exception(JText::_('COM_JROOMBOOKING_RESERVE_START_DATE_LESS_OR_EQUAL_END_DATE'));
        }

        return true;
    }

    /**
     * Metoda pro nahrani dat do formulare ze session
     *
     * @return array data formulare
     * @since 1.0.0
     */
    protected function loadFormData()
    {
        $data = JFactory::getApplication()->getUserState('com_jroombooking.jroombooking.filterdate.data', array());
        return $data;
    }
}