<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

class JRoomBookingRouter implements JComponentRouterInterface {


    /**
     * Prepare-method for URLs
     * This method is meant to validate and complete the URL parameters.
     * For example it can add the Itemid or set a language parameter.
     * This method is executed on each URL, regardless of SEF mode switched
     * on or not.
     *
     * @param   array $query An associative array of URL arguments
     *
     * @return  array  The URL arguments to use to assemble the subsequent URL.
     *
     * @since   1.0.0
     */
    public function preprocess($query)
    {
        return $query;
    }

    /**
     * Build method for URLs
     * This method is meant to transform the query parameters into a more human
     * readable form. It is only executed when SEF mode is switched on.
     *
     * @param   array &$query An array of URL arguments
     *
     * @return  array  The URL arguments to use to assemble the subsequent URL.
     *
     * @since   1.0.0
     */
    public function build(&$query)
    {

        $segments = array();
        if (isset($query['view']))
        {
            $segments[] = $query['view'];
            unset($query['view']);
        }
        if (isset($query['task']))
        {
            $segments[] = $query['task'];
            unset($query['task']);
        };
        if (isset($query['room_id']))
        {
            $segments[] = $query['room_id'];
            unset($query['room_id']);
        };
        return $segments;

    }

    /**
     * Parse method for URLs
     * This method is meant to transform the human readable URL back into
     * query parameters. It is only executed when SEF mode is switched on.
     *
     * @param   array &$segments The segments of the URL to parse.
     *
     * @return  array  The URL attributes to be used by the application.
     *
     * @since   1.0.0
     */
    public function parse(&$segments)
    {
        $vars = array();
        // Count segments
        $count = count($segments);
        // Handle View and Identifier
        switch ($segments[0])
        {
            case 'reserverequest':
                $vars['view'] = 'reserverequest';
                $vars['task'] = $segments[1];
                $id = explode(':', $segments[$count-1]);
                $vars['room_id'] = (int) $id[0];
                break;
            case 'reserveapply':
                $vars['view'] = 'reserveapply';
                $id = explode(':', $segments[$count-1]);
                $vars['room_id'] = (int) $id[0];
                break;
        }
        return $vars;
    }
}